import { useCallback, useContext, useState } from 'react';
import Button from 'react-bootstrap/Button';
import Form from 'react-bootstrap/Form';
import Modal from 'react-bootstrap/Modal';
import axios from 'axios';
import PropTypes from 'prop-types';
import { Buffer } from 'buffer/'; // note: the trailing slash is important!

import Web3Context from '../contexts/Web3Context';

function AddPostModal({
  show, setShow, title, postToBlockchain, onSubmit,
}) {
  const {
    provider, DAO, account,
  } = useContext(Web3Context);
  const [content, setContent] = useState('');

  const handleClose = () => setShow(false);

  const handleSubmit = useCallback(async () => {
    // Upload content to API
    // TODO: include metamask signature
    const msg = `0x${Buffer.from(content, 'utf8').toString('hex')}`;
    const signature = await provider.request({
      method: 'personal_sign',
      params: [msg, account],
    });
    const data = {
      author: account, content, signature,
    };
    const res = await axios.post('/api/write', data);
    const hash = res.data;
    setShow(false);
    setContent('');
    if (postToBlockchain) {
    // Upload hash to blockchain
      await DAO.methods.addPost(account, hash).send({
        from: account,
        gas: 1000000,
      });
    }
    if (onSubmit) {
      onSubmit(hash, data);
    }
  }, [provider, DAO, account, content, setShow, postToBlockchain, onSubmit]);

  return (
    <Modal show={show} onHide={handleClose}>
      <Modal.Header closeButton>
        <Modal.Title>{title}</Modal.Title>
      </Modal.Header>
      <Modal.Body>
        <Form>
          <Form.Group className="mb-3" controlId="addPost.content">
            <Form.Control
              as="textarea"
              rows={3}
              onChange={(e) => setContent(e.target.value)}
            />
          </Form.Group>
        </Form>

      </Modal.Body>
      <Modal.Footer>
        <Button variant="secondary" onClick={handleClose}>
          Close
        </Button>
        <Button variant="primary" onClick={handleSubmit}>
          Submit
        </Button>
      </Modal.Footer>
    </Modal>
  );
}

AddPostModal.propTypes = {
  show: PropTypes.bool.isRequired,
  setShow: PropTypes.func.isRequired,
  title: PropTypes.string,
  onSubmit: PropTypes.func,
  postToBlockchain: PropTypes.bool,
};

AddPostModal.defaultProps = {
  title: 'Add Post',
  onSubmit: undefined,
  postToBlockchain: false,
};

export default AddPostModal;
